# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tests/integration/schema_registry/data/proto/DependencyTestProto.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tests.integration.schema_registry.data.proto import NestedTestProto_pb2 as tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_NestedTestProto__pb2
from tests.integration.schema_registry.data.proto import PublicTestProto_pb2 as tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_PublicTestProto__pb2
try:
  tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_TestProto__pb2 = tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_PublicTestProto__pb2.tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_TestProto__pb2
except AttributeError:
  tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_TestProto__pb2 = tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_PublicTestProto__pb2.tests.integration.schema_registry.data.proto.TestProto_pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nFtests/integration/schema_registry/data/proto/DependencyTestProto.proto\x12$tests.integration.serialization.data\x1a\x42tests/integration/schema_registry/data/proto/NestedTestProto.proto\x1a\x42tests/integration/schema_registry/data/proto/PublicTestProto.proto\"\x98\x01\n\x11\x44\x65pendencyMessage\x12K\n\x0enested_message\x18\x01 \x01(\x0b\x32\x33.tests.integration.serialization.data.NestedMessage\x12\x11\n\tis_active\x18\x02 \x01(\x08\x12#\n\rtest_messsage\x18\x03 \x01(\x0b\x32\x0c.TestMessageB.\n,io.confluent.kafka.serializers.protobuf.testb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tests.integration.schema_registry.data.proto.DependencyTestProto_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,io.confluent.kafka.serializers.protobuf.test'
  _DEPENDENCYMESSAGE._serialized_start=249
  _DEPENDENCYMESSAGE._serialized_end=401
# @@protoc_insertion_point(module_scope)
