# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tests/integration/schema_registry/data/proto/PublicTestProto.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tests.integration.schema_registry.data.proto import TestProto_pb2 as tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_TestProto__pb2

from tests.integration.schema_registry.data.proto.TestProto_pb2 import *

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nBtests/integration/schema_registry/data/proto/PublicTestProto.proto\x12$tests.integration.serialization.data\x1a<tests/integration/schema_registry/data/proto/TestProto.protoP\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tests.integration.schema_registry.data.proto.PublicTestProto_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
