# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: test.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ntest.proto\x12\x04test\x1a google/protobuf/descriptor.proto\"\xc8\x02\n\x0bTestMessage\x12\x13\n\x0btest_string\x18\x01 \x01(\t\x12\x11\n\ttest_bool\x18\x02 \x01(\x08\x12\x12\n\ntest_bytes\x18\x03 \x01(\x0c\x12\x13\n\x0btest_double\x18\x04 \x01(\x01\x12\x12\n\ntest_float\x18\x05 \x01(\x02\x12\x14\n\x0ctest_fixed32\x18\x06 \x01(\x07\x12\x14\n\x0ctest_fixed64\x18\x07 \x01(\x06\x12\x12\n\ntest_int32\x18\x08 \x01(\x05\x12\x12\n\ntest_int64\x18\t \x01(\x03\x12\x15\n\rtest_sfixed32\x18\n \x01(\x0f\x12\x15\n\rtest_sfixed64\x18\x0b \x01(\x10\x12\x13\n\x0btest_sint32\x18\x0c \x01(\x11\x12\x13\n\x0btest_sint64\x18\r \x01(\x12\x12\x13\n\x0btest_uint32\x18\x0e \x01(\r\x12\x13\n\x0btest_uint64\x18\x0f \x01(\x04\x42\tZ\x07../testb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'test_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\007../test'
  _TESTMESSAGE._serialized_start=55
  _TESTMESSAGE._serialized_end=383
# @@protoc_insertion_point(module_scope)
