# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: example.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import confluent_kafka.schema_registry.confluent.meta_pb2 as meta_pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rexample.proto\x12\x04test\x1a\x14\x63onfluent/meta.proto\"\xab\x01\n\x06\x41uthor\x12\x16\n\x04name\x18\x01 \x01(\tB\x08\x82\x44\x05\x1a\x03PII\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x19\n\x07picture\x18\x03 \x01(\x0c\x42\x08\x82\x44\x05\x1a\x03PII\x12\r\n\x05works\x18\x04 \x03(\t\x12$\n\roneof_message\x18\x05 \x01(\x0b\x32\x0b.test.PizzaH\x00\x12 \n\x0coneof_string\x18\x06 \x01(\tB\x08\x82\x44\x05\x1a\x03PIIH\x00\x42\x0b\n\tpii_oneof\"\'\n\x05Pizza\x12\x0c\n\x04size\x18\x01 \x01(\t\x12\x10\n\x08toppings\x18\x02 \x03(\tB\tZ\x07../testb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'example_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\007../test'
  _AUTHOR.fields_by_name['name']._options = None
  _AUTHOR.fields_by_name['name']._serialized_options = b'\202D\005\032\003PII'
  _AUTHOR.fields_by_name['picture']._options = None
  _AUTHOR.fields_by_name['picture']._serialized_options = b'\202D\005\032\003PII'
  _AUTHOR.fields_by_name['oneof_string']._options = None
  _AUTHOR.fields_by_name['oneof_string']._serialized_options = b'\202D\005\032\003PII'
  _AUTHOR._serialized_start=46
  _AUTHOR._serialized_end=217
  _PIZZA._serialized_start=219
  _PIZZA._serialized_end=258
# @@protoc_insertion_point(module_scope)
