# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tests/integration/schema_registry/data/proto/SInt64Value.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n>tests/integration/schema_registry/data/proto/SInt64Value.proto\"\x1c\n\x0bSInt64Value\x12\r\n\x05value\x18\x01 \x01(\x12\x42.\n,io.confluent.kafka.serializers.protobuf.testb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tests.integration.schema_registry.data.proto.SInt64Value_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,io.confluent.kafka.serializers.protobuf.test'
  _SINT64VALUE._serialized_start=66
  _SINT64VALUE._serialized_end=94
# @@protoc_insertion_point(module_scope)
