# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tests/integration/schema_registry/data/proto/common_proto.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tests.integration.schema_registry.data.proto import metadata_proto_pb2 as tests_dot_integration_dot_schema__registry_dot_data_dot_proto_dot_metadata__proto__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?tests/integration/schema_registry/data/proto/common_proto.proto\x12\x0b\x43riteo.Glup\x1a\x41tests/integration/schema_registry/data/proto/metadata_proto.proto\"\xda\x01\n\x07\x43onsent\x12 \n\x18identification_forbidden\x18\x01 \x01(\x08\x12:\n\x06reason\x18\x02 \x01(\x0e\x32*.Criteo.Glup.IdentificationForbiddenReason\x12\x39\n\nset_fields\x18\xda\x86\x03 \x03(\x0b\x32#.Criteo.Glup.Consent.SetFieldsEntry\x1a\x30\n\x0eSetFieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01:\x04\x88\xb5\x18\x01*/\n\x16MarketingObjectiveType\x12\x08\n\x04Sale\x10\x00\x12\x0b\n\x07Install\x10\x01*\xd0\x01\n\x1dIdentificationForbiddenReason\x12\x0c\n\x08NoReason\x10\x00\x12\x1b\n\x17\x45xplicitConsentRequired\x10\x01\x12\x10\n\x0cOptoutCookie\x10\x02\x12\x13\n\x0f\x43toOptoutCookie\x10\x03\x12\x15\n\x11LimitedAdTracking\x10\x04\x12\x0e\n\nHstsOptout\x10\x05\x12\x14\n\x10\x44oNotTrackHeader\x10\x06\x12\r\n\tOoOCookie\x10\x07\x12\x11\n\rPendingOptout\x10\x08\x42\x11\n\x0f\x63om.criteo.glupb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tests.integration.schema_registry.data.proto.common_proto_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.criteo.glup'
  _CONSENT_SETFIELDSENTRY._options = None
  _CONSENT_SETFIELDSENTRY._serialized_options = b'8\001'
  _CONSENT._options = None
  _CONSENT._serialized_options = b'\210\265\030\001'
  _MARKETINGOBJECTIVETYPE._serialized_start=368
  _MARKETINGOBJECTIVETYPE._serialized_end=415
  _IDENTIFICATIONFORBIDDENREASON._serialized_start=418
  _IDENTIFICATIONFORBIDDENREASON._serialized_end=626
  _CONSENT._serialized_start=148
  _CONSENT._serialized_end=366
  _CONSENT_SETFIELDSENTRY._serialized_start=312
  _CONSENT_SETFIELDSENTRY._serialized_end=360
# @@protoc_insertion_point(module_scope)
